@@ -1,6 +1,7 @@
 /*
  * Copyright © 2002 Red Hat, Inc.
  * Copyright © 2008 Christian Persch
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * Mate-terminal is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -248,7 +249,7 @@ terminal_encoding_is_valid (TerminalEncoding *encoding)
 			                       " input  \"%s\"\n",
 			                       ascii_sample);
 			_terminal_debug_print (TERMINAL_DEBUG_ENCODINGS,
-			                       " output \"%s\" bytes read %u written %u\n",
+			                       " output \"%s\" bytes read %" G_GSIZE_FORMAT " written %" G_GSIZE_FORMAT "\n",
 			                       converted ? converted : "(null)", bytes_read, bytes_written);
 			if (error)
 				_terminal_debug_print (TERMINAL_DEBUG_ENCODINGS,
